#include "../../internal-include/BootLoader/boot_loader.hpp"
#include "../../internal-include/GPU/gpu_internal.hpp"
#include <PSX/System/IOPorts/dma_io.hpp>
#include <stdio.h>

#include <PSX/System/syscalls.hpp>

extern "C" uint32_t __heap_start;
extern "C" uint32_t __bss_start;
extern "C" uint32_t __bss_end;
extern "C" uint32_t __bss_len;
extern "C" uint32_t __planschi_start;
extern "C" uint32_t __planschi_end;

namespace JabyEngine {
    namespace boot {
        namespace Start {
            static void enable_DMA() {
                DMA_IO::DPCR.write(DMA_IO::DPCR.read().set(DMA_IO::DPCR::SPUEnable, DMA_IO::DPCR::GPUEnable, DMA_IO::DPCR::CDROMEnable));
            }

            static void setup() {
                static constexpr auto DebugX     = 1;
                static constexpr auto DebugY     = 0;
                static constexpr auto DebugScale = 1.0;

                __debug_boot_color_at(::JabyEngine::GPU::Color24::White(), DebugX, DebugY, DebugScale);
                enable_DMA();

                __debug_boot_color_at(::JabyEngine::GPU::Color24::Red(), DebugX, DebugY, DebugScale);
                SPU::stop_voices();
                
                __debug_boot_color_at(::JabyEngine::GPU::Color24::Green(), DebugX, DebugY, DebugScale);
                CD::setup();
                __debug_boot_color_at(::JabyEngine::GPU::Color24::Blue(), DebugX, DebugY, DebugScale);
                Timer::setup();

                __debug_boot_color_at(::JabyEngine::GPU::Color24::Yellow(), DebugX, DebugY, DebugScale);
                GPU::setup();
                GPU::display_logo();
                
                //Pause??
                SPU::setup();
            }
        }
    }

    static void test_bios_font() {
        static constexpr uint16_t SJIS = 0x83B5;

        const auto*      font     = SysCall::Krom2RawAdd(SJIS);
        const auto*const font_end = font + 16;
        printf("Loading SJIS from @0x%p\n", font);
        
        for(; font < font_end; font++) {
            const auto cur_char = __builtin_bswap16(*font);
            for(uint16_t shift = 1 << 15; shift; shift = shift >> 1) {
                printf(cur_char & shift ? "#" : " ");
            }

            printf("\n");
        }
    }

    void start() {
        static constexpr auto DebugX     = 0;
        static constexpr auto DebugY     = 0;
        static constexpr auto DebugScale = 1.0;

        __debug_boot_print_at(GPU::Color24::White(), DebugX, DebugY, DebugScale, "Starting Planschbecken\n");
        printf("Heap starts @0x%p\n", &__heap_start);
        printf("BSS      from 0x%p to 0x%p (%u)\n", &__bss_start, &__bss_end, __bss_len);
        test_bios_font();
        __debug_boot_print_at(GPU::Color24::Green(), DebugX, DebugY, DebugScale, "PLANSCHI from 0x%p to 0x%p\n", &__planschi_start, &__planschi_end);
        boot::Start::setup();
        
        __debug_boot_print_at(GPU::Color24::Red(), DebugX, DebugY, DebugScale, "Running main...\n");
        GPU::internal::wait_vsync(0);
        run();
    }
}