#pragma once
#include <PSX/File/file_types.hpp>
#include <PSX/GPU/gpu_types.hpp>
#include <PSX/GPU/make_gpu_primitives.hpp>

namespace JabyEngine {
    using FontPrimitive = GPU::SPRT::Linked;

    struct FontInfo {
        GPU::SizeU16 texture_size;
        GPU::SizeU16 font_size;
    };

    struct State {
        GPU::PositionI16 pos;
        uint8_t          wiggle_count;

        static constexpr State create(GPU::PositionI16 pos, uint8_t wiggle_count = 0) {
            return State{.pos = pos, .wiggle_count = wiggle_count};
        }
    };

    struct FontBufferInfo {
        FontPrimitive* double_buffer[2];
        size_t         single_buffer_length;

        static constexpr FontBufferInfo empty() {
            return FontBufferInfo{.double_buffer = {nullptr, nullptr}, .single_buffer_length = 0};
        }

        template<size_t N>
        static constexpr FontBufferInfo from(FontPrimitive (&buffer)[2][N]) {
            return FontBufferInfo{.double_buffer = {buffer[0], buffer[1]}, .single_buffer_length = N};
        }
    };

    using Wiggle = GPU::PositionI8[8];

    /*struct Wiggle {
        GPU::PositionI8 offset[8];
        uint8_t         count;

        GPU::PositionI8 next() {
            return this->offset[(this->count++)&0x7];
        }
    };*/
}
